# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Implements an Enum to prioritize subscribers in node discovery.
"""


class PriorityENUM:
    """
    The Enum to prioritize subscribers in node discovery.
    """
    LOW = 0
    HIGH = 2

    def __init__(self):
        pass



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

""" This module represents the Fault Tolerance Controller component from oneview_monasca agent."""

from oneview_monasca.shared import constants as const
from oneview_monasca.shared import log as logging
from oneview_monasca.shared import utils
from tooz import coordination

import time
import uuid

LOG = logging.get_logger(__name__)


class Coordinator(object):
    """
    Coordinator is the controller for multiple virtual machines that are running Monasca OneViewD.
    It is responsible to choosing an agent who was sleeping if the current metric publisher agent fall down.
    In this context, the publisher is called leader, because is the only who is really pushing metrics between all the
    coordinators running at this current time. The control of who is running is made by tooz, an OpenStack lib
    """

    def __init__(self, conf, taskto_run):
        """ This class implements a coordinator to run a specific method when it becomes a leader.

        :param conf: File containing the information necessary to start a new coordinator using tooz.
        :param taskto_run: Method that must be called when this Coordinator become the group leader.
        """
        utils.print_log_message('Info', 'Initialize Coordinator', LOG)
        self._conf = conf
        self._taskto_run = taskto_run
        self._coordinator_url = self._conf.coordinator_url
        self._id = self._generate_id()
        self._coordinator = coordination.get_coordinator(self._coordinator_url, self._id)
        self._group = self._conf.group_name
        self._stopped = False
        self._im_a_leader = False

    def start(self):
        """ Start the _coordinator, introduce it into a group, run the watchers in a new thread and callback the
        taskto_run when it is elected leader.
        """
        utils.print_log_message('Info', 'Initializing a new coordinator...', LOG)
        self._coordinator.start()
        self._coordinator.get_groups().get()
        self._coordinator.watch_join_group(self._group, self._join_group_callback)
        self._coordinator.watch_leave_group(self._group, self._leave_group_callback)
        self._coordinator.watch_elected_as_leader(self._group, self._leader_elected_callback)
        self._join_group()
        self._run_watchers()

    def stop(self):
        """ Stop the _coordinator if necessary and set the _stopped flag. """
        if not self._stopped:
            self._stopped = True
            self._coordinator.stop()

    @staticmethod
    def _generate_id():
        """ Generate and id for the _coordinator. """
        return '%s-%s' % (const.PREFIX_ID_COORD, uuid.uuid4())

    def _im_leader(self):
        """ Check if this coordinator is the current leader. """
        return self._im_a_leader

    def _join_group(self):
        """ Introduce the _coordinator for a group specified in the conf. If the group do not exists yet, this method
        create a new with the name specified.
        """
        if self._group not in self._coordinator.get_groups().get():
            self._coordinator.create_group(self._group).get()
        self._coordinator.join_group(self._group).get()

    def _join_group_callback(self, event):
        """ Callback to log when the _coordinator ingress the group. """
        message = 'The member {0} joined group {1}'.format(self._id, self._group)
        utils.print_log_message('Info', message, LOG)

    def _leader_elected_callback(self, event):
        """ Callback to log when a member is the new group leader and start a new thread calling back the taskto_run.
        :param event: A LeaderElected event
        """
        self._im_a_leader = True
        message = 'The member {0} is now running the Daemon'.format(event.member_id)
        utils.print_log_message('Info', message, LOG)
        self._taskto_run()

    @staticmethod
    def _leave_group_callback(event):
        """ Callback to log when a member leave the group.
        :param event: An instance of tooz.coordination.MemberJoinedGroup
        """
        message = 'The member {0} left group {1}'.format(event.member_id, event.group_id)
        utils.print_log_message('Info', message, LOG)

    def _run_watchers(self):
        """ Run tooz watchers for activities in groups. """
        while not self._stopped:
            self._coordinator.run_watchers()
            time.sleep(const.HEARTBEAT_INTERVAL)

    def step_down(self):
        """ Stand down as the group leader if we are.
        """
        self._coordinator.stand_down_group_leader(self._group)



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Creates and start a coordinator to start the application.
"""

from oneview_monasca.manager.manager_oneview import ManagerOneView
from oneview_monasca.manager.manager_monasca import ManagerMonasca
from oneview_monasca.eventbus.node_discovery import EventBUS
from oneview_monasca.eventbus.priority import PriorityENUM
from oneview_monasca.publisher.keeper import Keeper
from oneview_monasca.publisher.puller import Puller
from oneview_monasca.shared import log as logging
from oneview_monasca.publisher.scmb import SCMB
from oneview_monasca.shared import utils
from coordinator import Coordinator

import time

LOG = logging.get_logger(__name__)


class Daemon(object):
    """
    The Daemon is responsible to create and start a coordinator to start the application.
    The application will be started only when this coordinator is elected leader of the
    specific group in tooz/zookeeper for the Monasca/OneView. When it happens, a new
    SCMB object is called and it start to run the pushing actions to Monasca from
    OneView appliance, both specified in the configuration file.
    """
    def __init__(self, conf):
        utils.print_log_message('Info', 'Initialize Daemon', LOG)

        self._conf = conf
        self._coordinator = None
        self._eventbus = None
        self._keeper = None
        self._puller = None
        self._scmb = None

        # Setting debug mode
        self.debug = True if conf.DEFAULT.debug == 'true' else False

    def _start(self):
        """Initialized the publishers.
        """
        utils.print_log_message('Info', 'Creating EventBus', LOG)
        self.eventbus.start()

        utils.print_log_message('Info', 'Creating Publishers', LOG)
        # Create publishers.
        self.keeper.publish()
        self.puller.publish()
        self.scmb.publish()

        utils.print_log_message('Info', 'Subscribe Monasca publisher to intern publishers', LOG)
        # Subscribe intern publishers in Monasca publisher.
        self.scmb.subscribe(self.keeper)
        self.puller.subscribe(self.keeper)

        utils.print_log_message('Info', 'Subscribe publishers to eventbus', LOG)
        # Subscribe publisher to receive notification of eventbus.
        self.eventbus.subscribe(self.keeper, PriorityENUM.HIGH)
        self.eventbus.subscribe(self.scmb)
        self.eventbus.subscribe(self.puller)

    def _stop(self):
        """Restart the publishers.
        """
        time.sleep(5)
        utils.print_log_message('Info', 'Unsubscribe publishers to eventbus', LOG)
        # Unsubscribe publisher to receive notification of eventbus.
        self.eventbus.unsubscribe(self.puller)
        self.eventbus.unsubscribe(self.scmb)
        self.eventbus.unsubscribe(self.keeper)

        utils.print_log_message('Info', 'Unsubscribe Monasca publisher to intern publishers', LOG)
        # Unsubscribe intern publishers in Monasca publisher.
        self.puller.unsubscribe(self.keeper)
        self.scmb.unsubscribe(self.keeper)

        utils.print_log_message('Info', 'Stopping publishers and discarding its reference in Daemon', LOG)
        # Stopping publishers.
        self.keeper.stop()
        self.puller.stop()
        self.scmb.stop()
        # Discarding publishers reference in Daemon
        self.scmb = self.puller = self.keeper = None

        utils.print_log_message('Info', 'Oneview Monasca Daemon stopped, preparing to re-initialize', LOG)

    def _get_manager_oneview(self):
        """Get a instance of Manager Oneview
        """
        return ManagerOneView(
            host=self._conf.oneview.host,
            username=self._conf.oneview.username,
            password=self._conf.oneview.password,
            max_attempt=self._conf.DEFAULT.auth_retry_limit,
            certificates_directory=self._conf.DEFAULT.scmb_certificate_dir
        )

    def _get_manager_monasca(self):
        """Get a instance of Manager Monasca
        """
        return ManagerMonasca(
            auth_url=self._conf.openstack.auth_url,
            username=self._conf.openstack.auth_user,
            password=self._conf.openstack.auth_password,
            project_name=self._conf.openstack.auth_tenant_name,
            api_version=self._conf.openstack.monasca_api_version,
            debug=self.debug
        )

    @property
    def eventbus(self):
        """Get a instance of eventbus
        """
        if self._eventbus is None:
            self._eventbus = EventBUS(self._conf, debug=self.debug)

        return self._eventbus

    @property
    def keeper(self):
        """Get a instance of keeper
        """
        if self._keeper is None:
            self._keeper = Keeper(
                self._get_manager_oneview(),
                self._get_manager_monasca(),
                self._conf.DEFAULT.batch_publishing_interval,
                debug=self.debug
            )

        return self._keeper

    @keeper.setter
    def keeper(self, value):
        """Set keeper property with input value

        :param value: the new value of keeper property
        """
        self._keeper = value

    @property
    def puller(self):
        """Get a instance of puller
        """
        if self._puller is None:
            self._puller = Puller(
                self._get_manager_oneview(),
                self._conf.DEFAULT.periodic_refresh_interval,
                self.crash_callback,
                debug=self.debug
            )

        return self._puller

    @puller.setter
    def puller(self, value):
        """Set puller property with input value

        :param value: the new value of puller property
        """
        self._puller = value

    @property
    def scmb(self):
        """Get a instance of scmb
        """
        if self._scmb is None:
            self._scmb = SCMB(
                self._get_manager_oneview(),
                self._conf.oneview.host,
                self._conf.DEFAULT.auth_retry_limit,
                self.crash_callback,
                debug=self.debug
            )

        return self._scmb

    @scmb.setter
    def scmb(self, value):
        """Set scmb property with input value

        :param value: the new value of scmb property
        """
        self._scmb = value

    @property
    def coordinator(self):
        """Get a instance of coordinator to running fault tolerance
        """
        if self._coordinator is None:
            self._coordinator = Coordinator(self._conf.tooz, self._start)

        return self._coordinator

    @coordinator.setter
    def coordinator(self, value):
        """Set coordinator property with input value

        :param value: the new value of coordinator property
        """
        self._coordinator = value

    def start(self):
        """Starts the coordinator.
        """
        try:
            self.coordinator.start()
        except Exception as e:
            message = e.message + ": The agent is running without active/passive fault tolerance."
            utils.print_log_message('Warn', message, LOG)

            self.coordinator = None
            self._start()

    def crash_callback(self, exc_obj):
        """The function to restarting the agent server after fatal error in some publisher

        :param exc_obj: The exception object raised from a publisher
        """
        utils.print_log_message('Error', exc_obj, LOG)
        self._stop()

        if self._coordinator is not None:
            self.coordinator.step_down()
        else:
            self._start()



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Receives events from multiple plugins, sets the metrics and publishes them to listeners.
"""

from oneview_monasca.eventbus.priority import PriorityENUM
from oneview_monasca.shared import constants as const
from oneview_monasca.shared import log as logging
from oneview_monasca.shared import utils

import time
import copy

LOG = logging.get_logger(__name__)


class EventBUS(object):
    """
    This class is responsible for receiving events from multiple plugins,
    set the metrics by a single identifier (server_hardware_uuid) and publish to
    listeners.
    """
    _metaclass__ = utils.SingletonType

    def __init__(self, conf, debug=False):
        super(EventBUS, self).__init__()

        self._conf = conf
        self.debug = debug

        self._initialize()

    def _initialize(self):
        """
        Build data structure that will used.
        """
        self._drivers = {}
        self._subscribers = {
            PriorityENUM.LOW: set(),
            PriorityENUM.HIGH: set()
        }
        self._events = {}

    def _initialize_drivers(self):
        """
        Find extensions by namespace, initialize driver and start
        discover method in drivers found.
        """
        utils.print_log_message('Debug', 'Initialize Event BUS', LOG, self.debug)

        names = utils.list_names_driver(const.NAMESPACE_DISCOVERY_NODES)
        for name in names:
            utils.print_log_message('Info', 'Find driver [%s]' % name, LOG)
            try:
                if name not in self._drivers:
                    self._drivers[name] = utils.load_class_by_alias(
                        const.NAMESPACE_DISCOVERY_NODES,
                        name
                    )(self._conf, debug=self.debug)
                    self._drivers[name].subscribe(self)
                    self._drivers[name].discover()
            except ImportError:
                # Try load names found.
                utils.print_log_message('Info', 'Driver [%s] not found' % name, LOG)
            except Exception as ex:
                utils.print_log_message('Error', ex, LOG)
                raise

    def start(self):
        """
        Start drivers.
        """
        self._initialize_drivers()

    def stop(self):
        """
        Stop drivers.
        """
        try:
            for driver in self._drivers.values():
                driver.stop()

            # Clean the data structure.
            self._initialize()

        except Exception as e:
            utils.print_log_message('Error', e, LOG)

    def length_subscribers(self, priority=None):
        """
        Get length of the subscribers.

        :param priority: A PriorityENUM, the priority's value.
        :rtype: A :int: length by priority if not None otherwise all.
        """
        utils.print_log_message('Debug', 'length_subscribers, Priority[%s]' % priority, LOG)
        if priority is not None:
            return len(self._subscribers[priority])

        return len(self._subscribers[PriorityENUM.HIGH]) + len(self._subscribers[PriorityENUM.LOW])

    def _copy_event(self, node):
        """
        Ensure that it is Immutable.
        """
        if node.server_hardware_uuid in self._events:
            new_node = copy.deepcopy(self._events[node.server_hardware_uuid])
            # update set _events, adding metrics from node
            new_node.metrics.update(node.metrics)
        else:
            new_node = copy.deepcopy(node)

        self._events[node.server_hardware_uuid] = new_node

    def _resolve_available_metrics(self, nodes):
        """
        Group metrics by server hardware UUID.
        """
        updated_nodes = set()
        for node in nodes:
            self._copy_event(node)
            updated_nodes.add(self._events[node.server_hardware_uuid])

        return updated_nodes

    def _resolve_unavailable_metrics(self, nodes):
        """
        Verify and remove metrics of nodes that not should be monitored.
        """
        updated_nodes = set()
        for node in nodes:
            # Update set events, removing elements found in nodes
            self._events[node.server_hardware_uuid].metrics.difference_update(node.metrics)
            updated_nodes.add(self._events[node.server_hardware_uuid])

        return updated_nodes

    def available(self, nodes):
        """
        Notify subscribers with events of available nodes.
        """
        updated_nodes = self._resolve_available_metrics(nodes)
        if updated_nodes:
            # First notify subscribers of highest priority
            for subscribe in self._subscribers[PriorityENUM.HIGH]:
                subscribe.available(updated_nodes)

            time.sleep(1)

            # Second notify subscribers of lower priority
            for subscribe in self._subscribers[PriorityENUM.LOW]:
                subscribe.available(updated_nodes)

    def unavailable(self, nodes):
        """
        Notify subscribers with events of unavailable nodes.
        """
        updated_nodes = self._resolve_unavailable_metrics(nodes)
        # First notify subscribers of lower priority to not publish
        # in listener components of highest priority.
        for subscribe in self._subscribers[PriorityENUM.LOW]:
            subscribe.unavailable(updated_nodes)

        time.sleep(1)

        # Second notify subscribers of highest priority that will
        # not receive publication from listener components of lower priority.
        for subscribe in self._subscribers[PriorityENUM.HIGH]:
            subscribe.unavailable(updated_nodes)

    def subscribe(self, subscriber, priority=PriorityENUM.LOW):
        """
        Subscribe a subscriber by priority. Set priority low if is
        a listener component that publish information, otherwise if
        listener component is a information consumer, then set to high.
        """
        utils.print_log_message('Info', 'Subscribe %s with priority %s' % (subscriber, priority), LOG)
        self._subscribers[priority].add(subscriber)
        if len(self._events.values()) > 0:
            subscriber.available(set(self._events.values()))

    def unsubscribe(self, subscriber):
        """
        Unsubscribe a subscriber if subscribed.
        """
        for priority in self._subscribers.keys():
            if subscriber in self._subscribers[priority]:
                # removes subscriber from set subscribers[priority] if present
                self._subscribers[priority].discard(subscriber)



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

""" Manages the Monasca Component """

from monascaclient import client as monclient, ksclient
from oneview_monasca.manager.abstract_manager_monasca import AbstractManagerMonasca
from oneview_monasca.shared import log as logging
from oneview_monasca.shared import utils

import monascaclient.exc as exc

LOG = logging.get_logger(__name__)


class ManagerMonasca(AbstractManagerMonasca):
    """ The Monasca Manager """

    def __init__(self, auth_url, username, password, project_name, api_version, debug=False):
        super(ManagerMonasca, self).__init__()

        utils.print_log_message('Info', 'Initialize Monasca Manager', LOG)

        self.debug = debug
        self.auth_url = auth_url
        self.username = username
        self.password = password
        self.project_name = project_name
        self.api_version = api_version

    def _get_monasca_client(self):
        """Provide a Monasca client according a configuration file

        :returns: Monasca client.
        """
        message = "Using OpenStack credentials specified in the configuration file to get Monasca Client"
        utils.print_log_message('Debug', message, LOG, self.debug)

        # Authenticate to Keystone
        ks = ksclient.KSClient(
            auth_url=self.auth_url,
            username=self.username,
            password=self.password,
            project_name=self.project_name
        )
        # Monasca Client
        monasca_client = monclient.Client(
            self.api_version, ks.monasca_url, token=ks.token
        )
        return monasca_client

    def send_metrics(self, measurements):
        """ Update a list of metric dictionary  measurements to be in the
        Metric format that Monasca allow and sends them in one request.

        :param measurements: A list of Measurement objects to send to Monasca.
        """
        utils.print_log_message('Debug', 'Send Metric - method send_metrics', LOG, self.debug)
        monasca_metric_list = []

        for measure in measurements:
            new_metric = dict()
            new_metric['name'] = measure.name
            new_metric['value'] = measure.value
            new_metric['timestamp'] = measure.timestamp
            new_metric['dimensions'] = measure.dimensions
            new_metric['value_meta'] = measure.value_meta
            monasca_metric_list.append(new_metric)

        batch_metrics = {'jsonbody': monasca_metric_list}
        try:
            monasca_client = self._get_monasca_client()
            monasca_client.metrics.create(**batch_metrics)
        except exc.HTTPException as httpex:
            utils.print_log_message('Error', httpex.message, LOG)
        except Exception as ex:
            utils.print_log_message('Error', ex.message, LOG)
            raise

        utils.print_log_message('Info', 'Finished send metric - method send_metrics', LOG)



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
This module provide an Abstract Manager OneView that is a interface to create a
channel between the OneView Rest API and the publishers.
"""

import abc
import six


@six.add_metaclass(abc.ABCMeta)
class AbstractManagerOneView(object):
    """ Interface of a Manager OneView object.
    """
    @abc.abstractmethod
    def get_server_hardware_status(self, uuid, status):
        """ Get server hardware status and returns.

            :param uuid: The uuid of an specific server hardware.
            :param status: The current status associated an specific server hardware.

            :returns The status of corresponding server hardware.

        """
        raise NotImplementedError("Method not implemented, subclasses should implement this!")

    @abc.abstractmethod
    def get_server_hardware(self, uuid):
        """ Get a instance of a server hardware from oneview.

            :param uuid: the id associated oneview server hardware resource.
            :return: a instance of a oneview server hardware resource.
        """
        raise NotImplementedError("Method not implemented, subclasses should implement this!")

    @abc.abstractmethod
    def get_server_hardware_alerts(self, resource_uri, status):
        """Get the alerts associated an Oneview resource when it is not OK.

            :param resource_uri: the oneview uri resource.
            :param status: the status associated with input resource.

            :return: a dict that contains all alerts associated with input resource.
        """
        raise NotImplementedError("Method not implemented, subclasses should implement this!")

    @abc.abstractmethod
    def get_certificates(self):
        """ Get a hpOneView SDK connection.

            :return: a connection to communicate with Oneview Rest API.
        """
        raise NotImplementedError("Method not implemented, subclasses should implement this!")

    @abc.abstractmethod
    def validate_certificates(self):
        """ Validate the current SCMB certificates, if not expired.

        """
        raise NotImplementedError("Method not implemented, subclasses should implement this!")



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Subscribes the DiscoveryNode.
"""

import abc
import six


@six.add_metaclass(abc.ABCMeta)
class DiscoveryNodeSubscriber(object):
    """
    This class should be implemented by subscribers.
    """
    @abc.abstractmethod
    def available(self, nodes):
        """
        Notify subscribers with events of available nodes.
        """
        raise Exception("NotImplementedException")

    @abc.abstractmethod
    def unavailable(self, nodes):
        """
        Notify subscribers with events of unavailable nodes.
        """
        raise Exception("NotImplementedException")



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
This module is auxiliary function to check agent configuration file correctness
"""

from oneview_client import client
from ironicclient import client as ironic
from monascaclient import client as monclient, ksclient
from oneview_monasca.shared.section_read import SectionRead
from oneview_monasca.shared import log as logging
from oneview_monasca.shared import utils

import os
import re
import sys
import argparse
import traceback

LOG = logging.get_logger(__name__)


# TODO (diegoadolfo): Try move this class to exceptions module
class InvalidConfigFileException(Exception):
    """ The exception class that is raised when input config file is not valid.
    """
    def __init__(self, message=None):
        Exception.__init__(self, message)


def validate_config(config_file, shell=False):
    """ Function to test the correctness of agent configuration file

    :param config_file: The agent configuration file
    :param shell: Flag to identify if this script is running by shell
    """
    if config_file is None:
        raise InvalidConfigFileException("CONFIG FILE cannot be reachable.")

    # Type consistency: verify if fields are corrected filled with the specific expected type of data
    if False in map(_is_positive_int,
                    [config_file.DEFAULT.retry_interval, config_file.DEFAULT.auth_retry_limit,
                     config_file.DEFAULT.periodic_refresh_interval, config_file.DEFAULT.batch_publishing_interval]):
        raise InvalidConfigFileException("CONFIG FILE has any inconsistency type error, check it.")

    # Addresses: check every address to see if they are valid and they really lead to the correct machine
    if False in map(_is_url,
                    [config_file.oneview.manager_url, config_file.openstack.auth_url]):
        raise InvalidConfigFileException("CONFIG FILE has any invalid url, check it.")

    _chk_oneview_credentials(
        config_file.oneview.manager_url,
        config_file.oneview.username, config_file.oneview.password
    )

    _chk_openstack_credentials(
        config_file.openstack.auth_url, config_file.openstack.auth_user,
        config_file.openstack.auth_password, config_file.openstack.auth_tenant_name,
        config_file.openstack.monasca_api_version
    )

    try:
        _chk_ironic_credentials(
            config_file.ironic.auth_url, config_file.ironic.admin_user,
            config_file.ironic.admin_password, config_file.ironic.admin_tenant_name,
            config_file.ironic.insecure, config_file.ironic.ironic_api_version
        )
    except InvalidConfigFileException:
        raise
    except Exception:
        pass

    if shell:
        print('CONFIG FILE is valid, starting daemon application')
    else:
        utils.print_log_message('Info', 'CONFIG FILE is valid, starting daemon application', LOG)


def _is_positive_int(value):
    """Check if a string is a positive integer

    :param value: A string that will be checked.
    """
    if value[0] == '-':
        return False

    if value[0] == '+':
        return value[1:].isdigit()
    else:
        return value.isdigit()


def _is_url(value):
    """Check if a string is a valid url

    :param value: A string that will be checked.
    """
    regex = re.compile(
        r'^(?:http|ftp)s?://'  # http:// or https://
        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain...
        r'localhost|'  # localhost...
        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}|'  # ...or ipv4
        r'\[?[A-F0-9]*:[A-F0-9:]+\]?)'  # ...or ipv6
        r'(?::\d+)?'  # optional port
        r'(?:/?|[/?]\S+)$', re.IGNORECASE)

    return re.match(regex, value)


def _chk_ironic_credentials(auth_url, username, password, tenant_name, insecure, api_version):
    """Check if ironic credentials is valid

    :param auth_url: The ironic auth url
    :param username: The cloud-admin username
    :param password: The cloud-admin password
    :param tenant_name: The cloud-admin project name
    :param insecure: Flag to allow insecure requests
    :param api_version: The ironic api version
    """
    kwargs = {
        'os_username': username,
        'os_password': password,
        'os_auth_url': auth_url,
        'os_tenant_name': tenant_name,
        'os_ironic_api_version': api_version
    }
    if insecure.lower() == 'true':
        kwargs['insecure'] = True

    try:
        ironic.get_client(1, **kwargs)
    except:
        raise InvalidConfigFileException(
            "Impossible created a connection with ironic services, check the CONFIG FILE")


def _chk_openstack_credentials(auth_url, username, password, project_name, api_version):
    """Check if openstack credentials is valid

    :param auth_url: The keystone auth url
    :param username: The cloud-admin username
    :param password: The cloud-admin password
    :param project_name: The cloud-admin project name
    :param api_version: The monasca api version
    """
    try:
        ks = ksclient.KSClient(auth_url=auth_url, username=username, password=password, project_name=project_name)
        monclient.Client(api_version, ks.monasca_url, token=ks.token)
    except:
        raise InvalidConfigFileException(
            "Impossible created a connection with openstack services, check the CONFIG FILE")


def _chk_oneview_credentials(manager_url, username, password, max_attempt=20, ca_file='', insecure='True'):
    """Check if oneview credentials is valid

    :param manager_url: The oneview appliance host
    :param username: The administrator username
    :param password: The administrator password
    """
    kwargs = {
        'manager_url': manager_url,
        'username': username,
        'password': password,
        'max_polling_attempts': max_attempt,
    }

    if ca_file:
        kwargs['tls_cacert_file'] = ca_file
    if insecure.lower() == 'true':
        kwargs['allow_insecure_connections'] = True

    try:
        client.ClientV2(**kwargs)
    except:
        raise InvalidConfigFileException(
            "Impossible created a connection with oneview services, check the CONFIG FILE")


def main():
    """
    The main method
    """
    parser = argparse.ArgumentParser(
        add_help=True,
        formatter_class=argparse.RawTextHelpFormatter,
        description='''This script will be validate a user input config file to run Monasca-Oneviewd'''
    )
    # Global Arguments
    parser.add_argument(
        '-c', '--config-file',
        dest='config_file', required=False,
        help='''Default path to configuration file'''
    )

    args = parser.parse_args()
    full_path_to_file = os.path.realpath(os.path.expanduser(args.config_file))

    config_file = SectionRead(full_path_to_file)
    try:
        validate_config(config_file, True)
    except:
        traceback.print_exc()


if __name__ == '__main__':
    main()
    sys.exit(0)



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
This module represents the Puller component from oneview_monasca agent.
"""

from oneview_monasca.eventbus.base import DiscoveryNodeSubscriber
from oneview_monasca.publisher.base import PublisherProvider
from oneview_monasca.shared import log as logging
from oneview_monasca.model.status import Status
from oneview_monasca.shared import utils
from threading import Thread
from threading import Lock

import time

LOG = logging.get_logger(__name__)


class Puller(PublisherProvider, DiscoveryNodeSubscriber, Thread):
    """ The Puller is a publisher for Metric Keeper and a Subscriber from a Node
    Discovery. It process status of OneView resources and publish to
    subscribers.

    Thread control:
        stopped: Manage the thread state (running or stopped).
        lock: Manage the access of another publishers to shared data structs to avoid race condition.
    """

    def __init__(self, manager_oneview, refresh_interval, crash_callback, debug=False):
        super(Puller, self).__init__()
        Thread.__init__(self)

        self.debug = debug
        self._manager_oneview = manager_oneview
        self._crash_callback = crash_callback
        self._refresh_interval = int(refresh_interval)

        #  This boolean indicates if it's the first time that the Puller
        #  receives available nodes.
        self._first_available_iteration = True
        self._monitored_nodes = set()

        # Thread attributes control
        self._lock = Lock()
        self._stopped = True

    def stop(self):
        """ Stop the Thread.
        """
        self._stopped = True
        self._Thread__stop()

    def publish(self):
        """ Initialize the process of Component.
        """
        utils.print_log_message('Info', 'Initialize Puller', LOG)

        self._stopped = False
        self.start()

    def available(self, nodes):
        """ Receive a set of nodes to be monitored.

        :param nodes: A set of Node objects.
        """
        updated_servers_hardware = set()
        for node in nodes:
            updated_servers_hardware.add(node.server_hardware_uuid)
            utils.log_actions(node, "discovered", self.debug, LOG)

        self._lock.acquire()
        self._monitored_nodes.update(updated_servers_hardware)
        self._lock.release()

        if self._first_available_iteration:
            self._first_available_iteration = False
            self._process_status()

    def unavailable(self, nodes):
        """ Receive a set of nodes and very if these nodes should be removed
        from monitored nodes.

        To verify, the puller looks at metrics in nodes: if metrics is empty,
        it will remove the node, else it will keep the node in monitored nodes.

        :param nodes: A set of Node objects.
        """
        updated_servers_hardware = set()
        for node in nodes:
            if not node.metrics:
                updated_servers_hardware.add(node.server_hardware_uuid)
                utils.log_actions(node, "removed", self.debug, LOG)

        self._lock.acquire()
        self._monitored_nodes.difference_update(updated_servers_hardware)
        self._lock.release()

    def _process_status(self):
        """ Process status of each OneView resource in the set of monitored
        nodes, creating a Status object and, at last publish a set of statuses
        for subscribers.
        """
        utils.print_log_message('Info', 'Start process status from OneView resources', LOG)

        try:
            states = set()
            # Locking shared resource
            self._lock.acquire()
            for server_hardware_uuid in self._monitored_nodes:
                status, str_timestamp = self._manager_oneview.get_server_hardware_status(server_hardware_uuid)

                if status is not None and str_timestamp:
                    modified_timestamp = utils.parse_timestamp(str_timestamp)
                    states.add(Status(server_hardware_uuid, status, modified_timestamp))

            # Unlocking shared resource
            self._lock.release()

            if states:
                self.status_update(states)
                utils.print_log_message('Info', 'End process status from OneView resources', LOG)

        except Exception as ex:
            self._lock.release()
            self._crash_callback(ex)

    def run(self):
        """ Pull and process status from OneView resources and publish to
        subscribers.
        """
        while True:
            if not self._stopped:
                self._process_status()

            time.sleep(self._refresh_interval)



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Command-line interface to the Monasca OneView Daemon.
"""

from __future__ import print_function

from oneview_monasca.infrastructure.commands import do_genconfig
from oneview_monasca.shared.section_read import SectionRead
from oneview_monasca.infrastructure import commands as config
from oneview_monasca.infrastructure import validations
from oneview_monasca.shared import constants as const
from oneview_monasca.shared import utils
from builtins import input

import argparse
import os

VERSION = '1.0'

COMMAND_MODULES = [config]


class ParseCommand(object):
    """Class that implements the command-line interface to the Monasca OneView Daemon.
    """
    def __init__(self):
        super(ParseCommand, self).__init__()
        self.parser = None
        self.subcommands = None

    @staticmethod
    def get_base_parser():
        """Gets the base parser of the Monasca OneView Daemon.
        """
        epilog = 'See "{app_name} --help COMMAND for help on a specific command.'.format(
            app_name=const.APPLICATION_NAME
        )

        parser = argparse.ArgumentParser(
            prog=const.APPLICATION_NAME,
            description=__doc__.strip(),
            epilog=epilog,
            add_help=False,
            formatter_class=HelpFormatter,
        )

        # Global arguments
        parser.add_argument('-v', '--version', action='version',
                            version=VERSION)
        parser.add_argument('-h', '--help', action='help',
                            help=argparse.SUPPRESS)
        parser.add_argument('-c', '--config-file',
                            help='Default path to configuration file')

        return parser

    def get_subcommand_parser(self):
        """Gets the subcommand parser ot the Monasca OneView Daemon.
        """
        parser = self.get_base_parser()
        self.subcommands = {}

        subparsers = parser.add_subparsers(metavar='<subcommand>')
        self.enhance_parser(subparsers, self.subcommands)
        self.define_commands_from_module(subparsers, self, self.subcommands)
        return parser

    @utils.arg('command', metavar='<subcommand>', nargs='?', help='Display help for <subcommand>')
    def do_help(self, args):
        """Displays a help about this program or one of its subcommands.

        :param args: name of the program or subcommands.
        """
        if getattr(args, 'command', None):
            if args.command in self.subcommands:
                self.subcommands[args.command].print_help()
            else:
                raise Exception("'%s' is not a valid subcommand" % args.command)
        else:
            self.parser.print_help()

    def define_commands_from_module(self, subparsers, command_module, cmd_mapper):
        """Adds *do_* methods in a module and add as node_discoverer into a subparsers.

        :param subparsers: subparser collection.
        :param command_module: name of the module where the command is defined.
        :param cmd_mapper: TO DO
        """
        for method_name in (a for a in dir(command_module) if a.startswith('do_')):
            # Commands should be hypen-separated instead of underscores.
            command = method_name[3:].replace('_', '-')
            callback = getattr(command_module, method_name)
            self.define_command(subparsers, command, callback, cmd_mapper)

    def enhance_parser(self, subparsers, cmd_mapper):
        """Enhances a parser.

        :param subparsers: collection of parsers.
        :param cmd_mapper: TODO (documentation)
        """
        for command_module in COMMAND_MODULES:
            self.define_commands_from_module(subparsers, command_module, cmd_mapper)

    @staticmethod
    def define_command(subparsers, command, callback, cmd_mapper):
        """Define a command in the subparsers collection.

        :param subparsers: subparsers collection where the command will go.
        :param command: command name.
        :param callback: function that will be used to process the command.
        :param cmd_mapper: TODO (documentation)
        """

        desc = callback.__doc__ or ''
        help = desc.strip().split('\n')[0]
        arguments = getattr(callback, 'arguments', [])

        subparser = subparsers.add_parser(
            command,
            help=help,
            description=desc,
            add_help=False,
            formatter_class=HelpFormatter
        )
        subparser.add_argument('-h', '--help', action='help', help=argparse.SUPPRESS)
        cmd_mapper[command] = subparser

        for (args, kwargs) in arguments:
            subparser.add_argument(*args, **kwargs)

        subparser.set_defaults(func=callback)

    def parse(self, argv):
        """Parses an argument.
        :param argv: name of the argument.
        """
        parser = self.get_base_parser()
        (options, args) = parser.parse_known_args(argv)
        subcommand_parser = self.get_subcommand_parser()
        self.parser = subcommand_parser

        if len(argv) == 0 or '-c' in argv or '--config-file' in argv:
            config_file = self.get_configuration(options)
            validations.validate_config(config_file)
            return config_file

        args = subcommand_parser.parse_args(argv)
        args.func(args)

    @staticmethod
    def get_configuration(options):
        """Gets a configuration based on given options.

        :param options: collection of options names.
        """
        file_name = options.config_file

        if file_name:
            full_path_to_file = os.path.realpath(os.path.expanduser(file_name))
            return SectionRead(full_path_to_file)
        else:
            while True:
                create = utils.get_input("Config file not found. "
                                         "Would you like to create one now? [Y/n]") or 'y'

                if create.lower() == 'y':
                    do_genconfig(options)
                    # Loading new config file
                    full_path_to_file = os.path.realpath(os.path.expanduser(file_name))
                    return SectionRead(full_path_to_file)
                elif create.lower() == 'n':
                    return None
                else:
                    print("Invalid option.\n")


class HelpFormatter(argparse.HelpFormatter):
    """Intermediate class to HelpFormatter.
    :param argparser.HelpFormatter: performs the text formatting.
    """

    def start_section(self, heading):
        """Formats a section to begin with upper case.
        :param heading: section to be formatted
        """
        # Title-case the headings
        heading = '%s%s' % (heading[0].upper(), heading[1:])
        super(HelpFormatter, self).start_section(heading)



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Gathers useful methods for the whole system.
"""

from builtins import input
from oneview_monasca.shared.exceptions import LoginFailException
from oneview_monasca.shared import log as logging
from stevedore import extension
from datetime import datetime
from stevedore import driver
from retrying import retry

import os
import re
import pytz
import time
import tzlocal
import urlparse

LOG = logging.get_logger(__name__)


def print_log_message(log_type, message, log=LOG, debug=False):
    """Encapsulation of the logging message.

    :param log_type: the level of  logging that will be applied.
    :param message: the message to be presented into the logging.
    :param log: the log object from class that call this method.
    :param debug: flag to enable debug mode.
    :raises ValueError if the type of logging is invalid.
    """
    log_type = log_type.lower().strip()
    if log_type == 'debug' and debug:
        log.debug(message)
    elif log_type == 'info':
        log.info(message)
    elif log_type == 'error':
        log.error(message)
    elif log_type == 'warn':
        log.warn(message)
    elif log_type != 'debug':
        raise ValueError('Invalid log type: ' + log_type + '.')


def extract_domain_from_service_url(url):
    """Extracts the domain of a service url.

    :param url: the url to be extracted.
    :raises Exception if the received url has an invalid format.
    :returns the domain of the service url.
    """
    regex = re.compile(
        r'^(?:http|ftp)s?://'  # http:// or https://
        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain...
        r'localhost|'  # localhost...
        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}|'  # ...or ipv4
        r'\[?[A-F0-9]*:[A-F0-9:]+\]?)'  # ...or ipv6
        r'(?::\d+)?'  # optional port
        r'(?:/?|[/?]\S+)$', re.IGNORECASE)

    if not re.match(regex, url):
        print_log_message('Error', 'Invalid URL input')
        raise Exception('Invalid URL input')
    else:
        return urlparse.urlparse(url).hostname


def makedirs(path, mode=0o777):
    """Creates a directory if it does not exist.
    :param path: path to the deserted directory.
    :param mode: the permission that a directory will be created.
    :raises OSError if the directory path is invalid.
    """

    if not path:
        raise OSError("Not a valid directory path")
    elif path[0] != os.path.sep and path[0] != '~':
        path = os.path.expanduser('~') + os.path.sep + path

    _makedirs(path, mode)


def _makedirs(path, mode=0o777):
    """Creates a directory if it does not exist.

    :param path: path to the deserted directory.
    :param mode: the permission that a directory will be created.
    :raises OSError if there is no permission to create the file.
    :returns None if the directory already exists.
    """
    if not path or os.path.exists(path):
        return

    head, tail = os.path.split(path)
    _makedirs(head, mode)

    try:
        os.makedirs(path, mode)
    except OSError as ex:
        message = 'OSError: %(errno)d, %(error)s, %(file)s. Permission denied to create the certificates directory.' % {
            'errno': ex.errno, 'error': ex.strerror, 'file': ex.filename
        }
        print_log_message('Error', message)
        raise


def save_file(directory, filename, certificate):
    """Saves a file with a certificate.
    :param directory: name of the directory where the file should be saved.
    :param filename: name of the file to be saved.
    :param certificate: certificate that should be written in the file.
    :returns the path to the saved file
    """
    if not os.path.exists(directory):
        makedirs(directory)

    pathfile = directory + filename

    with open(pathfile, 'w+') as f:
        f.write(certificate)
    return pathfile


def extract_oneview_uuid(oneview_uri):
    """Function to extract the uuid associated to OneView resource

    :param oneview_uri: The url with the uuid from the OneView resource
    :return: The uuid for the OneView resource
    """
    ov_uuid = oneview_uri.split('/')[-1]
    return ov_uuid


def load_class_by_alias(namespace, name, invoke_args=(), invoke_on_load=False, log=True):
    """Load class using stevedore alias.

    :param namespace: A string, namespace where the alias is defined.
    :param invoke_args: An object, the configuration to driver.
    :param name: A string, alias name of the class to be loaded.
    :param invoke_on_load: A boolean, if should invoke on load.
    :param log: enable log message
    :returns class if calls can be loaded.
    :raises ImportError if class cannot be loaded.
    """
    try:
        if log:
            message = 'Load class by alias: Namespace[%s], Name[%s].' % (namespace, name)
            print_log_message('Info', message)

        class_to_load = driver.DriverManager(
            namespace=namespace,
            name=name,
            invoke_args=invoke_args,
            invoke_on_load=invoke_on_load
        ).driver

    except RuntimeError:
        raise ImportError("Class not found.")

    return class_to_load


def list_names_driver(namespace, log=True):
    """Get a list of the names from namespace.

    :param namespace: namespace where the alias is defined.
    :param log: enable log message
    :returns list of names to the namespace.
    :raises Exception if any error.
    """
    try:
        if log:
            print_log_message('Info', 'Get names by Namespace[%s].' % namespace)

        ext = extension.ExtensionManager(namespace=namespace)
        return ext.names()
    except RuntimeError:
        raise Exception("Error list names.")


def parse_timestamp(str_timestamp, format='%Y-%m-%dT%H:%M:%S.%fZ'):
    """Parses a time stamp.

    :param str_timestamp: a string, the timestamp.
    :param format: a string, the default is date timezone.
    :returns the parsed time stamp in millis seconds.
    """
    local_tz = tzlocal.get_localzone()
    date_time = datetime.strptime(str_timestamp, format)
    date_time = date_time.replace(tzinfo=pytz.utc).astimezone(local_tz)
    timestamp = time.mktime(date_time.timetuple()) + date_time.microsecond / 1E6

    return int(timestamp * 1000)


def not_retry_if_login_fail(exception):
    """Function to check if a LoginFailException occurs.

    :param exception: Some exception class
    :return: True if the input exception not is a instance of LoginFailException
    """
    return not isinstance(exception, LoginFailException)


def try_execute(func, max_attempt, wait, *args):
    """Function that trying to execute another function max_attempt times.

        :param func: A function that will be executed.
        :param max_attempt: The number max of attempts to execute input function.
        :param wait: The time (in milliseconds) between each attempt of execute input function.
        :param args: A list of params to rum input function.

        :returns the input function output.
        :raises Exception if burst the max attempts.
    """
    @retry(wait_fixed=wait, stop_max_attempt_number=max_attempt, retry_on_exception=not_retry_if_login_fail)
    def execute():
        """ Auxiliary function to use retry decorator.

        :return: the input function output.
        :raises: Exception if burst the max attempts.
        """
        try:
            return func(*args)
        except LoginFailException:
            raise
        except:
            print_log_message('Info', " ---- Trying Again ---- ")
            raise

    # Executing the input function by retry decorator
    return execute()


class SingletonType(type):
    """A simply type that has only one interesting inhabitant.
    """
    def __call__(cls, *args, **kwargs):
        """Calls the class.
        """
        try:
            return cls.__instance
        except AttributeError:
            cls.__instance = super(
                SingletonType, cls).__call__(*args, **kwargs)
            return cls.__instance


def arg(*args, **kwargs):
    """Decorator for CLI args.

    Example:

    >>> @arg("name", help="Name of the new entity")
    ... def entity_create(args):
    ...     pass
    """
    def _decorator(func):
        """ Auxiliary function to use decorator for CLI args.

        :param func: A function that will be executed.
        :return: A function decorator
        """
        add_arg(func, *args, **kwargs)
        return func
    return _decorator


def add_arg(func, *args, **kwargs):
    """Bind CLI arguments to a shell.py `do_foo` function."""

    if not hasattr(func, 'arguments'):
        func.arguments = []

    # NOTE(sip): avoid dips that can occur when the module is shared across
    # tests.
    if (args, kwargs) not in func.arguments:
        # Because of the semantics of decorator composition if we just append
        # to the options list positional options will appear to be backwards.
        func.arguments.insert(0, (args, kwargs))


def extract_metric_names(metrics):
    """ Receive metrics to extract only the names
    and return these names in a list.

    :param metrics: Metrics to be processed.
    """
    names = []
    for metric in metrics:
        names.append(metric.name)

    return names


def log_actions(node, action, debug=False, log=LOG):
    """ This method send logs when a node is discovered or removed.

    :param node: Node that was discovered or removed.
    :param action: String representing the action.
    :param log: the log object from class that call this method.
    :param debug: flag to enable debug mode.
    """
    message_debug = 'Node %s: %s with metrics: %s' % (
        action,
        node.server_hardware_uuid,
        node.metrics
    )
    print_log_message('Debug', message_debug, log, debug)

    message_info = 'Node %s: %s with metrics: %s' % (
        action,
        node.server_hardware_uuid,
        extract_metric_names(node.metrics)
    )
    print_log_message('Info', message_info, log)


def get_input(message):
    """
    Gets an input from the user.
    :param message: the message that indicates what the user should input
    :return: the user input
    """
    return input(message)



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


"""Generates the configuration file based on the user input.
"""

from oneview_monasca.shared import utils as utils
from oneview_monasca.shared import constants as const
from configparser import ConfigParser

import os
import getpass


def do_genconfig(args):
    """Generates the config file according to user input
    """

    print("========= DEFAULT ========")
    debug = utils.get_input(
        "Enable agent in debug mode [y/N]: ") or 'n'
    retry_interval = utils.get_input(
        "Type the polling interval in seconds for daemon to manage the nodes: ")
    batch_publishing_interval = utils.get_input(
        "Type the publishing interval in seconds for daemon to push the metrics: ")
    refresh_interval = utils.get_input(
        "Type the polling interval in seconds to get health status directly from OneView: ")
    scmb_certificate_dir = utils.get_input(
        "Type the certificates directory to register in OneView SCMB [/var/run/oneview-monasca]: ")
    auth_retry_limit = utils.get_input(
        "Type the maximum number of attempts to try authenticate in REST API: ")

    debug = 'false' if debug == 'n' else 'true'
    retry_interval = retry_interval if retry_interval else "300"
    refresh_interval = refresh_interval if refresh_interval else "180"
    batch_publishing_interval = batch_publishing_interval if batch_publishing_interval else "60"

    auth_retry_limit = auth_retry_limit if auth_retry_limit else "5"
    scmb_certificate_dir = scmb_certificate_dir if scmb_certificate_dir else "/var/run/oneview-monasca"

    scmb_certificate_dir = os.path.realpath(os.path.expanduser(scmb_certificate_dir))
    utils.makedirs(scmb_certificate_dir)

    print("========= Openstack =========")
    auth_url = utils.get_input("Type the Keystone url for authentication: ")
    auth_user = utils.get_input("Type the name of your OpenStack user: ")
    auth_password = getpass.getpass("Type the password for your OpenStack user: ")
    auth_tenant_name = utils.get_input("Type the tenant name that the OpenStack user will be authenticated: ")
    monasca_api_version = utils.get_input("Type a version of Monasca API that you want to use [2_0]: ")

    monasca_api_version = monasca_api_version if monasca_api_version else "2_0"

    print("========= OneView =========")
    oneview_manager_url = utils.get_input("Type the manager_url for the OneView services: ")
    oneview_username = utils.get_input("Type your OneView username: ")
    oneview_password = getpass.getpass("Type your OneView user's password: ")
    oneview_insecure = utils.get_input("Would you like to allow insecure connections to OneView? [Y/n]: ") or "Y"
    max_polling_attempts = utils.get_input("Max polling attempts OneView requests: ")
    tls_cacert_file = utils.get_input("Path to your CA OneView certificate file, if any: ")

    oneview_host = utils.extract_domain_from_service_url(oneview_manager_url)
    oneview_insecure = "true" if oneview_insecure.lower() == 'y' else "false"
    max_polling_attempts = max_polling_attempts if max_polling_attempts else "15"

    fault_tolerance_enable = False
    group_name = coordinator_url = None
    while True:
        create = utils.get_input("Would you like to enable fault tolerance in the agent? [Y/n] ") or 'y'

        if create.lower() == 'y':
            print("========= Tooz =========")

            group_name = utils.get_input("The group name for tooz configuration: ")
            coordinator_url = utils.get_input("The coordinator url for tooz configuration: ")
            fault_tolerance_enable = True
            break
        elif create.lower() == 'n':
            break
        else:
            print("Invalid option.\n")

    config_drivers = {}
    try:
        names = utils.list_names_driver(const.NAMESPACE_DISCOVERY_NODES, log=False)
    except Exception as ex:
        print('\nCannot load installed drivers - Error caused by %s\n' % str(ex))
        names = []

    for name in names:
        try:
            conf = utils.load_class_by_alias(
                const.NAMESPACE_DISCOVERY_NODES, name, log=False).genconfig()

            config_drivers[name.split('_')[-1]] = conf
        except Exception as ex:
            print('\nCannot generating config file session to driver: %s - Error caused by %s\n' % (name, str(ex)))

    # Write Configuration file #
    config = ConfigParser()
    config.set("DEFAULT", "debug", debug)
    config.set("DEFAULT", "retry_interval", retry_interval)
    config.set("DEFAULT", "periodic_refresh_interval", refresh_interval)
    config.set("DEFAULT", "batch_publishing_interval", batch_publishing_interval)

    config.set("DEFAULT", "auth_retry_limit", auth_retry_limit)
    config.set("DEFAULT", "scmb_certificate_dir", scmb_certificate_dir)

    if fault_tolerance_enable:
        config.add_section("tooz")
        config.set("tooz", "group_name", group_name)
        config.set("tooz", "coordinator_url", coordinator_url)

    config.add_section("openstack")
    config.set("openstack", "auth_url", auth_url)
    config.set("openstack", "auth_user", auth_user)
    config.set("openstack", "auth_password", auth_password)
    config.set("openstack", "auth_tenant_name", auth_tenant_name)
    config.set("openstack", "monasca_api_version", monasca_api_version)

    config.add_section("oneview")
    config.set("oneview", "host", oneview_host)
    config.set("oneview", "manager_url", oneview_manager_url)
    config.set("oneview", "username", oneview_username)
    config.set("oneview", "password", oneview_password)
    config.set("oneview", "allow_insecure_connections", oneview_insecure)
    config.set("oneview", "max_polling_attempts", max_polling_attempts)
    config.set("oneview", "tls_cacert_file", tls_cacert_file)

    for driver in config_drivers:
        config.add_section(driver)
        for option, value in config_drivers[driver].items():
            config.set(driver, option, value)

    if not args.config_file:
        args.config_file = '~' + os.path.sep + 'oneview_monasca.conf'

    filename = utils.get_input(
        "Type the path of the new configuration file [%s]: " % args.config_file) or args.config_file
    full_filename = os.path.realpath(os.path.expanduser(filename))

    config_dir = os.path.dirname(full_filename)
    utils.makedirs(config_dir)

    with open(full_filename, 'w') as configfile:
        config.write(configfile)
        print("======\nFile created successfully on '%s'!\n======" % filename)



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Default exception class' that is used for the publishers and managers.
"""


class LoginFailException(Exception):
    """ The exception class that is raised when fail login occurs.
    """
    def __init__(self, message=None):
        Exception.__init__(self, message)


class HTTPFailException(Exception):
    """ The exception class that is raised when HTTP error occurs.
    """
    def __init__(self, message=None):
        Exception.__init__(self, message)


class SCMBConnectionFailException(Exception):
    """ The exception class that is raised when the SCMB class cannot connection with RabbitMQ.
    """
    def __init__(self, message=None):
        Exception.__init__(self, message)


class InvalidConfigFileException(Exception):
    """ The exception class that is raised when input config file is not valid.
    """
    def __init__(self, message=None):
        Exception.__init__(self, message)



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
This class represents a OneView resource status.
"""


class Status(object):
    """ Init method for Status class.

    :param server_hardware_uuid: A string, the UUID from the Oneview resource.
    :param status: A int, the value of status in server hardware.
    :param modified_timestamp: A date timezone, the timestamp receive from the message.
    """
    def __init__(self, server_hardware_uuid, status, modified_timestamp):
        self.server_hardware_uuid = server_hardware_uuid
        self.status = status
        self.modified_timestamp = modified_timestamp

    def __eq__(self, other):
        if isinstance(other, Status):
            return self.server_hardware_uuid == other.server_hardware_uuid and \
                self.status == other.status and \
                self.modified_timestamp == other.modified_timestamp
        return False

    def __hash__(self):
        return hash(self.server_hardware_uuid)

    def __repr__(self):
        return 'server_hardware_uuid[%s], status[%s], modified_timestamp[%s]' % (
            self.server_hardware_uuid,
            self.status,
            self.modified_timestamp
        )



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Global constants used in whole project
"""


''' APPLICATION '''
APPLICATION_NAME = 'monasca_oneviewd'

''' EventBUS '''
# The namespace to drivers of node discovery.
NAMESPACE_DISCOVERY_NODES = 'node_discovery.driver'

''' METRICS '''
METRIC_NAME = "oneview.node_status"

'''' MANAGER ONEVIEW '''
# Map with possibles status form the server hardware
METRIC_VALUE_PARSER = {
    "OK": 0,
    "Disabled": 1,
    "Critical": 2,
    "Warning": 3,
    "Unknown": 4
}
MAX_VALUE_META_LEN = 13
ONEVIEW_URI_PREFIX = '/rest/server-hardware/'

''' LOG '''
# The format to output log
FORMATTER_LOG = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'

'''COORDINATOR'''
# A prefix name to the id of each coordinator created by tooz.
PREFIX_ID_COORD = b'oneviewd'
# Time to send a signal to inform that the coordinator is alive
HEARTBEAT_INTERVAL = 1

''' SCMB LISTENER '''
# The SCMB protocol name.
EXCHANGE_NAME = "scmb"
# The default routing key the listener all server hardware updates.
ROUTING_KEY = "scmb.server-hardware.Updated.#"
# The host to connect Pika and RabbitMQ.
MB_PORT = 5671

''' ONEVIEW MANAGER '''
# The base url to get a OneView alert.
ALERT_BASE_URL = 'Active' + "'&filter=resourceUri='"
# The message to invalid login
LOGIN_FAILED = 'Invalid username or password or directory.'
# The message if certificate is still valid
CERTIFICATE_VALID = 'The existing certificate is still valid'
# Error 400
HTTP_ERROR_400 = 'response: 400'
# Resource not found in oneview
RESOURCE_NOT_FOUND = 'Resource not found'



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
This module represents the Keeper component from oneview_monasca agent.
"""

from oneview_monasca.eventbus.base import DiscoveryNodeSubscriber
from oneview_monasca.model.measurement import Measurement
from oneview_monasca.publisher.base import PublisherSubscriber
from oneview_monasca.shared import log as logging
from oneview_monasca.shared import utils
from threading import Thread
from threading import Lock

import time

LOG = logging.get_logger(__name__)


class Keeper(PublisherSubscriber, DiscoveryNodeSubscriber, Thread):
    """
    This class represents the Keeper component from oneview_monasca
    agent.

    The Keeper it's responsible for store and public all newer health
    status measurements from OneView resources received from any of its
    publishers, all metrics are published into Monasca.

    Thread control:
        stopped: Manage the thread state (running or stopped).
        lock: Manage the access of another publishers to shared data structs to avoid race condition.
    """

    def __init__(self, oneview_manager, monasca_manager, batch_time, debug=False):
        super(Keeper, self).__init__()
        Thread.__init__(self)

        self.debug = debug
        self._metric_storage = {}
        self._batch_time = int(batch_time)

        self._manager_oneview = oneview_manager
        self._manager_monasca = monasca_manager

        # Thread attributes control
        self._lock = Lock()
        self._stopped = True

    def stop(self):
        """ Stops the Thread """
        self._stopped = True
        self._Thread__stop()

    def publish(self):
        """ Start the  Keeper """
        utils.print_log_message('Info', 'Initialize Keeper', LOG)

        self._stopped = False
        self.start()

    def status_update(self, states):
        """
        This method update the data structure with all newer status
        information from nodes, and pushes all this new information to
        Monasca.
        """
        valid_metrics = []
        for status in states:
            uuid = status.server_hardware_uuid
            updated = self._update_status(status)
            if updated:
                self._update_meta(uuid)
                self._lock.acquire()
                node_metrics = self._create_measurements(uuid)
                self._lock.release()
                valid_metrics.extend(node_metrics)

        if len(valid_metrics) > 0:
            self._manager_monasca.send_metrics(valid_metrics)
        else:
            utils.print_log_message('Info', 'There is no metrics to be sent', LOG)

    def _update_status(self, status):
        """
        This method update the Keeper data structure for a given status if
        valid.

        :param status: A Status object representing a server hardware state from OneView.
        :rtype: A :boolean: - True, if the server hardware have a newer status.
        """
        result = False
        uuid = status.server_hardware_uuid

        # Locking shared resource
        self._lock.acquire()
        if uuid in self._metric_storage:
            stored_status = self._metric_storage[uuid].get('status')

            if stored_status is None:
                self._metric_storage[uuid]['status'] = status
                utils.print_log_message('Info', "gathered first status for %s:%s" % (uuid, status), LOG)
                result = True

            elif status.modified_timestamp >= stored_status.modified_timestamp:
                self._metric_storage[uuid]['status'] = status
                result = True

        # Unlocking shared resource
        self._lock.release()
        return result

    def _update_meta(self, uuid):
        """
        This method update the meta value information for a given uuid.

        :param uuid: The server hardware uuid from monitored OneView resource
        """
        # Locking shared resource
        self._lock.acquire()

        status = self._metric_storage[uuid]['status'].status
        self._metric_storage[uuid]['meta'] = \
            self._manager_oneview.get_server_hardware_alerts(uuid, status)

        # Unlocking shared resource
        self._lock.release()

    def _create_measurements(self, uuid):
        """
        This method create a list of measurements for a given uuid.

        :param uuid: The server hardware uuid from monitored OneView resource
        :rtype: A list with all Measurements objects for the given uuid.
        """
        measurements_list = []
        for metric in self._metric_storage[uuid]['metrics']:
            stored_status = self._metric_storage[uuid].get('status')
            if stored_status is not None:
                meta = self._metric_storage[uuid]['meta']
                value = stored_status.status

                measurement = Measurement(metric.name, value, metric.dimensions, meta)
                measurements_list.append(measurement)

        return measurements_list

    def available(self, nodes):
        """
        This method updates the data structure adding new nodes or new
        information about nodes that are already stored.
        """
        # Locking shared resource
        self._lock.acquire()

        for node in nodes:
            if node.server_hardware_uuid not in self._metric_storage:
                utils.log_actions(node, "discovered", self.debug, LOG)
                self._metric_storage[node.server_hardware_uuid] = {
                    'metrics': node.metrics, 'status': None, 'meta': {}
                }
            else:
                self._metric_storage[node.server_hardware_uuid]['metrics'] = node.metrics

        # Unlocking shared resource
        self._lock.release()

    def unavailable(self, nodes):
        """
        This method updates the data structure removing nodes or information
        about nodes that are already stored.
        """
        # Locking shared resource
        self._lock.acquire()

        for node in nodes:
            del self._metric_storage[node.server_hardware_uuid]

        # Unlocking shared resource
        self._lock.release()

    def run(self):
        """
        This method pull all stored information in data structure and send to
        Monasca
        """
        while True:
            time.sleep(self._batch_time)

            if not self._stopped:
                try:
                    all_measurements = []
                    # Locking shared resource
                    self._lock.acquire()
                    for uuid in self._metric_storage:
                        measurements = self._create_measurements(uuid)
                        all_measurements.extend(measurements)

                    # Unlocking shared resource
                    self._lock.release()
                    self._manager_monasca.send_metrics(all_measurements)
                    utils.print_log_message(
                        'Debug', 'Finished send actual metrics from data structure', LOG, self.debug)
                except Exception as ex:
                    utils.print_log_message('Error', "Keeper failed: %s" % ex.message, LOG)



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
This module provide the Initialize of the Monasca OneView Daemon
"""

from oneview_monasca.infrastructure.validations import InvalidConfigFileException
from oneview_monasca.shared.parse_command import ParseCommand
from oneview_monasca.application.daemon import Daemon
from oneview_monasca.shared import log as logging
from oneview_monasca.shared import utils

import os
import sys
import signal

LOG = logging.get_logger(__name__)


def main():
    """The main method that starts the daemon execution.

    :raises a KeyboardInterrupt if the user interrupts the execution via keyboard.
    :raises an Exception if something else goes wrong.
    """
    try:
        conf = ParseCommand().parse(sys.argv[1:])
        if conf:
            Daemon(conf).start()
    except KeyboardInterrupt:
        print("\nOneView Monasca Daemon stopped")
        sys.exit(130)
    except InvalidConfigFileException as confEx:
        print("\nInvalid config file input. OneView Monasca Daemon stopped")
        utils.print_log_message('Error', confEx, LOG)
        sys.exit(1)
    except Exception as ex:
        print("\nUnexpected error. OneView Monasca Daemon stopped")
        utils.print_log_message('Error', ex, LOG)
        os.kill(os.getpid(), signal.SIGTERM)


if __name__ == '__main__':
    main()



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""Provides for publish components.
"""
import abc
import six


@six.add_metaclass(abc.ABCMeta)
class PublisherProvider(object):
    """
    This class represent a provider for publisher components.
    """
    def __init__(self):
        super(PublisherProvider, self).__init__()
        self.subscribers = {}

    def subscribe(self, subscriber):
        """Subscribe a listener.
        :param subscriber: the listener component.
        """
        key = subscriber.__class__.__name__
        if key not in self.subscribers.keys():
            self.subscribers[key] = subscriber

    def unsubscribe(self, subscriber):
        """Unsubscribe a listener
        :param subscriber: the listener component.
        """
        key = subscriber.__class__.__name__
        if key in self.subscribers.keys():
            del self.subscribers[key]

    def status_update(self, states):
        """
        Send a set of Status from OneView resources.
        :param states: Set of :class: `Status` with new about OneView resources.
        """
        for subscriber in self.subscribers.values():
            subscriber.status_update(states)


@six.add_metaclass(abc.ABCMeta)
class PublisherSubscriber(object):
    """
    This class represent a subscriber for publisher components.
    """
    def __init__(self):
        super(PublisherSubscriber, self).__init__()

    @abc.abstractmethod
    def status_update(self, states):
        """
        Receive a set of Status from OneView resources.
        :param states: Set of :class: `Status` with new about OneView resources.
        """
        raise Exception("NotImplementedException")



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Reads a section of a file.
"""

import configparser


class SectionRead:
    """Class that implements the reading of a section of a file.
    """
    def __init__(self, full_path_to_file, defaults={}):
        self._CONF = configparser.ConfigParser(defaults)
        self._CONF.readfp(open(full_path_to_file))

    def __getattr__(self, section):
        """Gets an attribute based on a section of a file.
        :param section: section to be analyzed.
        """
        result = Section(self._CONF.items(section), section)
        return result

    def __nonzero__(self):
        return True


class Section(object):
    """Class that represents a section of a file.
    """
    def __init__(self, cfg_section, section):
        self.__dict__ = dict(cfg_section)
        self.section = section

    def __getattribute__(self, *args, **kwargs):
        """Gets an attribute based on a section.
        :param args: arguments to match in the file.
        :param kwargs: TODO (documentation)
        """
        try:
            return object.__getattribute__(self, *args, **kwargs)
        except AttributeError:
            raise AttributeError("Missing required attribute '%s' on "
                                 "section '%s' " % (args[0], self.section))



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
This module represents the SCMB component from oneview_monasca agent.
"""

from oneview_monasca.shared.exceptions import SCMBConnectionFailException
from oneview_monasca.eventbus.base import DiscoveryNodeSubscriber
from oneview_monasca.shared.exceptions import LoginFailException
from oneview_monasca.publisher.base import PublisherProvider
from oneview_monasca.shared import constants as const
from oneview_monasca.shared import log as logging
from pika.credentials import ExternalCredentials
from oneview_monasca.model.status import Status
from pika.exceptions import AMQPConnectionError
from pika.exceptions import AMQPChannelError
from oneview_monasca.shared import utils
from threading import Thread
from threading import Lock

import json
import pika

LOG = logging.get_logger(__name__)


class SCMB(PublisherProvider, DiscoveryNodeSubscriber, Thread):
    """
    The SCMB class to provider a RabbitMQ messages callbacks

    Thread control:
        stopped: Manage the thread state (running or stopped).
        lock: Manage the access of another publishers to shared data structs to avoid race condition.
    """
    def __init__(self, manager_oneview, host, max_retry_attempts, crash_callback, debug=False):
        super(SCMB, self).__init__()
        Thread.__init__(self)

        self.debug = debug
        # Agent attributes usage
        self._host = host
        self._monitored_nodes = set()
        self._crash_callback = crash_callback
        self._manager_oneview = manager_oneview
        self._max_retry_attempts = int(max_retry_attempts)

        # Thread attributes control
        self._lock = Lock()
        self._stopped = True

        # RabbitMQ attributes manage
        self._channel = None
        self._connection = None
        self._reload_certs = True

    def stop(self):
        """ Stop the Thread.
        """
        self._stop_scmb()
        self._stopped = True

        self._Thread__stop()

    def publish(self):
        """ Initialize the process of Component.
        """
        utils.print_log_message('Info', 'Initialize SCMB', LOG)

        self._stopped = False
        self.start()

    def available(self, nodes):
        """ Receive a set of nodes to be monitored.

        :param nodes: A set of Node objects.
        """
        updated_nodes = set()
        for node in nodes:
            updated_nodes.add(node.server_hardware_uuid)
            utils.log_actions(node, "discovered", self.debug, LOG)

        self._lock.acquire()
        self._monitored_nodes.update(updated_nodes)
        self._lock.release()

    def unavailable(self, nodes):
        """ Receive a set of nodes and very if these nodes should be removed
        from monitored nodes.

        To verify, the puller looks at metrics in nodes: if metrics is empty,
        it will remove the node, else it will keep the node in monitored nodes.

        :param nodes: A set of Node objects.
        """
        updated_nodes = set()
        for node in nodes:
            if not node.metrics:
                updated_nodes.add(node.server_hardware_uuid)
                utils.log_actions(node, "removed", self.debug, LOG)

        self._lock.acquire()
        self._monitored_nodes.difference_update(updated_nodes)
        self._lock.release()

    @property
    def connection(self):
        """ Private function to create a connection with RabbitMQ using the Pika library.

        :return: A instance of pika connection
        """
        if self._connection is None:
            # Validate current certificate
            self._manager_oneview.validate_certificates()

            # Setup our ssl options
            ssl_options = self._manager_oneview.get_certificates()

            # Connect to RabbitMQ
            self._connection = pika.BlockingConnection(
                pika.ConnectionParameters(
                    self._host,
                    const.MB_PORT,
                    credentials=ExternalCredentials(),
                    ssl=True,
                    ssl_options=ssl_options
                )
            )
        return self._connection

    @connection.setter
    def connection(self, value):
        if self._connection is not None and self._connection.is_open:
            utils.print_log_message('Info', 'Closing connection with RabbitMQ', LOG)
            self._connection.close()

        # Set connection to None value
        self._connection = value

    def _initialize_scmb(self):
        """ Function to open a communication channel between the Monasca/OneViewD and OneView SCMB.

        :raises: A instance of SCMBConnectionFailException when burst the max_attempts to connect with RabbitMQ
        """
        utils.print_log_message('Info', 'Starting agent for listening changes in monitored nodes', LOG)

        # Create a connection with RabbitMQ and Create and bind to queue
        self._channel = self.connection.channel()

        result = self._channel.queue_declare()
        queue_name = result.method.queue

        self._channel.queue_bind(exchange=const.EXCHANGE_NAME, queue=queue_name, routing_key=const.ROUTING_KEY)

        self._channel.basic_consume(self._scmb_callback, queue=queue_name, no_ack=True)

        utils.print_log_message('Info', 'Start listening for SCMB messages', LOG)
        self._reload_certs = True
        self._channel.start_consuming()

    def _stop_scmb(self):
        """ Shutdown the connection to Oneview SCMB by stopping the consumer with RabbitMQ.
        When RabbitMQ confirms the cancellation, on_cancelok
        will be invoked by pika, which will then closing the channel and
        connection.
        """

        # Stop listening for messages
        if self._channel is not None:
            utils.print_log_message('Info', 'Stopping the RabbitMQ channel', LOG)
            self._channel.stop_consuming()

        self._channel = None
        self.connection = None
        utils.print_log_message('Info', 'Connection with RabbitMQ was stopped successfully', LOG)

    def on_cancelok(self, unused_frame):
        """ This method is invoked by pika when RabbitMQ acknowledges the cancellation of a consumer.

        :param unused_frame:
        """
        utils.print_log_message('Info', 'Closing the RabbitMQ channel', LOG)
        self._channel.close()

    def run(self):
        """ Default method the start a thread
        """
        while not self._stopped and self._max_retry_attempts > 0:
            try:
                self._initialize_scmb()
            except (LoginFailException, OSError) as ex:
                self._stopped = True
                self._crash_callback(ex)
            except (AMQPConnectionError, AMQPChannelError) as ex:
                self._retry_reconnect(ex)
            except Exception as ex:
                self._retry_reconnect(ex, 0)

        if self._max_retry_attempts == 0:
            self._crash_callback(
                SCMBConnectionFailException('Fail to open a connection with SCMB, fixed it and trying again')
            )

    def _get_status(self, resource_uuid, str_status, str_timestamp):
        """ Function the get the status associated a node resource

        :param resource_uuid: The UUID associated a node resource
        :param str_status: the status associated a input node resource
        :param str_timestamp: the timestamp of status input
        :return: a Status object
        """
        try:
            status = self._manager_oneview.get_server_hardware_status(resource_uuid, str_status)
            # Converting timestamp
            timestamp = utils.parse_timestamp(str_timestamp)

            return Status(resource_uuid, status, timestamp)
        except Exception as ex:
            self._crash_callback(ex)

    def _scmb_callback(self, ch, method, properties, body):
        """ Function to SCMB callbacks

        :param ch: the channel to connect SCMB and RabbitMQ
        :param method: the method used to get the messages
        :param properties: the message properties
        :param body: the body of message
        """
        # Parsing State-Change Message Bus message body
        message = json.loads(body)
        # Get interest resource
        resource = message['resource']

        message = 'Pull metric to resource with key %(resource_key)s' % {
            "resource_key": method.routing_key
        }
        utils.print_log_message('Info', message, LOG)

        self._lock.acquire()
        if resource['uuid'] not in self._monitored_nodes:
            message = 'Resource %(uuid)s not found in current monitored, waiting for node discoverer update' % {
                'uuid': resource['uuid']
            }
            utils.print_log_message('Info', message, LOG)

        else:
            uuid = resource['uuid']
            # Get Resource status and timestamp
            status, timestamp = resource['status'], resource['modified']
            # Pulling Status Metric
            self.status_update({self._get_status(uuid, status, timestamp)})

        self._lock.release()

    def _retry_reconnect(self, exc_obj, mode=1):
        """ Function to try reconnect agent with SCMB when a exception is raised

        :param exc_obj: the raised exception
        :param mode: If mode equal to one try re-validate SCMB certificates, else apply default behavior
        """
        self._stop_scmb()
        utils.print_log_message('Error', exc_obj, LOG)

        if self._reload_certs and mode:
            self._reload_certs = False
            utils.print_log_message(
                'Info', "Connection with SCMB closed, trying validate the certificates again", LOG)
            return

        self._max_retry_attempts -= 1
        if self._max_retry_attempts > 0:
            utils.print_log_message('Info', "Trying reconnect to OneView SCMB", LOG)



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
This module provide an Manager OneView that is the concrete class of Abstract
OneView Manager that is used to create a channel between the OneView Rest API
and the publishers.
"""

from oneview_monasca.shared.exceptions import LoginFailException
from oneview_monasca.shared.exceptions import HTTPFailException
from abstract_manager_oneview import AbstractManagerOneView
from oneview_monasca.shared import constants as const
from hpOneView import connection, security, activity
from hpOneView.exceptions import HPOneViewException
from hpOneView.oneview_client import OneViewClient
from oneview_monasca.shared import log as logging
from oneview_monasca.shared import utils

import ssl

LOG = logging.get_logger(__name__)


class ManagerOneView(AbstractManagerOneView):
    """
    The concrete Manager OneView class
    """
    def __init__(self, host, username, password, max_attempt=0, certificates_directory=None):
        super(ManagerOneView, self).__init__()

        self._host = host
        self._username = username
        self._password = password
        self._max_attempt = int(max_attempt)
        self._directory = certificates_directory

    def get_server_hardware_status(self, uuid, status=None):
        """ Get server hardware status and returns

        :param uuid: The uuid of an specific server hardware.
        :param status: The current status associated an specific server hardware.

        :returns The status of corresponding server hardware.
        """
        if status:
            return const.METRIC_VALUE_PARSER[status]
        else:
            server_hardware = self.get_server_hardware(uuid)
            if server_hardware:
                status, timestamp = server_hardware['status'], server_hardware['modified']

                return const.METRIC_VALUE_PARSER[status], timestamp

        return None, None

    def get_server_hardware(self, uuid):
        """Call the function get_server_hardware encapsulate into run_by_retry function

        :param uuid: the id associated oneview server hardware resource.

        :return: a instance of a oneview server hardware resource.
        :raise LoginFailException if a client is no authenticated in Oneview Rest Api.
        :raises Exception if burst the max attempts.
        """
        return self._run_by_retry(self._get_server_hardware, uuid)

    def get_server_hardware_alerts(self, resource_uri, status):
        """Call the function get_server_hardware_alerts encapsulate into run_by_retry function

        :param resource_uri: the oneview uri resource
        :param status: the status associated with input resource.

        :return: a dict that contains all alerts associated with input resource.
        :raise LoginFailException if a client is no authenticated in Oneview Rest Api.
        :raises Exception if burst the max attempts.
        """
        return self._run_by_retry(self._get_server_hardware_alerts, resource_uri, status)

    def get_connection(self):
        """Call the function get_connection encapsulate into run_by_retry function

        :return: a connection to communicate with Oneview Rest Api
        :raise LoginFailException if a client is no authenticated in Oneview Rest Api.
        :raises Exception if burst the max attempts.
        """
        return self._run_by_retry(self._get_connection)

    def get_activity(self):
        """ Get a hpOneView activity model

        :return: a instance of activity model from hpOneView SDK.
        """
        con = self.get_connection()
        return activity(con)

    def get_security(self):
        """ Get a hpOneView security model

        :return: a instance of security model from hpOneView SDK.
        """
        con = self.get_connection()
        return security(con)

    def get_certificates(self):
        """ Get the path to SCMB certificates to open a connection with RabbitMQ

        :return: a dict that contains all certificates path
        """

        client_security = self.get_security()

        # Get a OneView certificates to connect on SCMB
        ca = client_security.get_cert_ca()
        kp = client_security.get_rabbitmq_kp()

        # Get pathfile certificates
        caroot = utils.save_file(self._directory, '/caroot.pem', ca)
        keydat = utils.save_file(self._directory, '/key.pem', kp['base64SSLKeyData'])
        client = utils.save_file(self._directory, '/client.pem', kp['base64SSLCertData'])

        return {
            "keyfile": keydat,
            "ca_certs": caroot,
            "certfile": client,
            "server_side": False,
            "cert_reqs": ssl.CERT_REQUIRED
        }

    def validate_certificates(self):
        """Call the function validate_certificates encapsulate into run_by_retry function

        :raise LoginFailException if a client is no authenticated in Oneview Rest Api.
        :raises Exception if burst the max attempts.
        """
        return self._run_by_retry(self._validate_certificates)

    def _get_server_hardware(self, uuid):
        """ Get a instance of a server hardware from oneview

        :param uuid: the id associated oneview server hardware resource.
        :return: a instance of a oneview server hardware resource.
        """
        client = OneViewClient({
            'ip': self._host,
            'credentials': {'userName': self._username, 'password': self._password}
        })
        try:
            return client.server_hardware.get(uuid)
        except HPOneViewException as hpex:
            if const.RESOURCE_NOT_FOUND in str(hpex):
                utils.print_log_message('Info', hpex.msg.replace('\n', ' '), LOG)
                return {}
            else:
                raise

    def _get_server_hardware_alerts(self, resource_uuid, status):
        """Get the alerts associated an Oneview resource when it is not OK

        :param resource_uuid: the server hardware uuid for the resource
        :param status: the status associated with input resource.

        :return: a dict that contains all alerts associated with input resource.
        """
        if status is None or status == const.METRIC_VALUE_PARSER['OK']:
            return {}

        act = self.get_activity()
        oneview_uri = const.ONEVIEW_URI_PREFIX + resource_uuid
        alerts = act.get_alerts(const.ALERT_BASE_URL + oneview_uri)

        meta_values = {}
        for alert in alerts:
            if len(meta_values) < const.MAX_VALUE_META_LEN:
                meta_values[alert['uri']] = self._host + '#/activity/r' + alert['uri']
            else:
                break

        return meta_values

    def _get_connection(self):
        """ Get a hpOneView SDK connection

        :return: a connection to communicate with Oneview Rest Api
        """
        con = connection(self._host)
        con.login({'userName': self._username, 'password': self._password})

        if con.get_eula_status():
            con.set_eula(supportAccess='no')

        return con

    def _validate_certificates(self):
        """ Validate the current SCMB certificates

        :return: None
        """
        try:
            client_security = self.get_security()
            client_security.gen_rabbitmq_self_signed_ca()
        except Exception as ex:
            if const.CERTIFICATE_VALID in str(ex):
                utils.print_log_message('Info', 'The certificate does not need to be validated.', LOG)
            else:
                raise ex

    def _run_by_retry(self, func, *args):
        """Function to manager Exceptions between Oneview Rest Api connection.

        :param func: A function that will be executed.
        :param args: A list of params to rum input function.

        :returns the input function output.
        :raise LoginFailException if a client is no authenticated in Oneview Rest Api.
        :raises Exception if burst the max attempts.
        """
        try:
            return utils.try_execute(func, self._max_attempt, 2000, *args)
        except HPOneViewException as hpex:
            if const.HTTP_ERROR_400 in str(hpex):
                utils.print_log_message('Error', const.HTTP_ERROR_400, LOG)
                raise HTTPFailException(const.HTTP_ERROR_400)

            elif const.LOGIN_FAILED in str(hpex):
                utils.print_log_message('Error', const.LOGIN_FAILED, LOG)
                raise LoginFailException(const.LOGIN_FAILED)

            else:
                msg = 'Unexpected error: %(message)s' % {'message': str(hpex)}
                utils.print_log_message('Error', msg, LOG)
                raise hpex
        except:
            raise



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
This module represent a measurement object that ManagerMonasca
will use.
"""

import time


class Measurement(object):
    """
    This class represents the measurement that will be used by
    ManagerMonasca to extract all information and create a metric
    to send to Monasca.

    :param name: (string(255), required) - The name of the metric.
    :param value: (float, required) - Value of the metric. Values with base-10
    exponents greater than 126 or less than -130 are truncated.
    :param dimensions: ({string(255): string(255)}, optional) - A dictionary
    consisting of (key, value) pairs used to uniquely identify a metric.
    :param value_meta: ({string(255): string}(2048), optional) - A dictionary
    consisting of (key, value) pairs used to add information about the value.
    Value_meta key value combinations must be 2048 characters or less
    including '{"":""}' 7 characters total from every json string.
    """
    def __init__(self, name, value, dimensions={}, value_meta={}):
        self.name = name
        self.value = value
        self.timestamp = time.time() * 1000
        self.dimensions = dimensions
        self.value_meta = value_meta

    def __eq__(self, other):
        """Method to compare Measurement objects """
        if isinstance(other, Measurement):
            return self.value == other.value and \
                self.dimensions.items() == other.dimensions.items() and \
                self.value_meta.items() == other.value_meta.items()
        return False



# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Generates LOGs for actions.
"""

import oneview_monasca.shared.constants as const

import logging
import sys

# Register loggers
LOGGERS = {}


def _get_handler():
    """A simple handler to the standard output.
    """
    handler = logging.StreamHandler(sys.stdout)
    handler.setFormatter(
        logging.Formatter(const.FORMATTER_LOG)
    )

    return handler


def get_logger(name):
    """Create a logger configuration.

    Usage::

        >>> import log
        >>> LOG = log.get_logger(__name__)
        >>> LOG.info(...)

    :param name: The class name.
    :rtype: A :class:`Logger <Logger>`
    """
    if name not in LOGGERS:
        logger = logging.getLogger(name)
        logger.setLevel(logging.DEBUG)
        logger.addHandler(_get_handler())
        LOGGERS[name] = logger

    return LOGGERS[name]
